{***********************************************************************}
{                                                                       }
{           Copyright (C) 1996 - 1997 by Artem A. Berman                }
{           E-mail: art@aber.kherson.ua                                 }
{                                                                       }
{***********************************************************************}
{ This components can be freely used and distributed in commercial and  }
{ private environments, provied this notice is not modified in any way. }
{ ----------------------------------------------------------------------}
{ Version 1.0.  Date last modified:  05/10/96                           }
{ Version 1.01  Date last modified:  16-Mar-97                          }
{    + Fixed bug with Sorted property, now it works in the correct way. }
{    + Fixed bug when error occuired when you tried to press enter and  }
{      no one Item has been selected                                    }
{    + Under Win 32 names of folders/files are case sensitive now       }
{    + StringCase property - case of the stings in the TExplorerBox     }
{    + Builds directory structure faster                                }
{ Version 1.02 2-Apr-97                                                 }
{    + Now it is really "like explorer" control. Added icons for files. }
{ Version 1.03 7-Apr-97                                                 }
{    + Added 16 bit version. Even there it is "like Explorer" control.  }
{    + Fixed bug with filters.                                          }
{ ----------------------------------------------------------------------}

unit Explorer;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, StdCtrls, ShellApi,
  Controls, Menus, Forms;

type
  TStringsCase = (cs_Normal, cs_Upper, cs_Lower);

  TExplorerBox = class(TCustomListBox)
  private
    FFileName,
    FDirectory: TFileName;
    FBitmaps: array [0..3] of TBitmap;
    FOnFileChange,
    FOnDirChange: TNotifyEvent;
    FFilter: string;
    FStringsCase: TStringsCase;
    FIcon: TIcon;
    procedure SetDirectory(Value: TFileName);
    procedure SetStringsCase(ACase: TStringsCase);
  protected
    procedure DrawItem(Index: Integer; Recty: TRect; State: TOwnerDrawState); override;
    procedure Update; virtual;
    procedure Click; override;
    procedure DblClick; override;
    procedure RePaint; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function RPos(const SubStr: Char; Str: string): Word;
  published
    property Directory: TFileName read FDirectory write SetDirectory;
    property FileName: TFileName read FFileName;
    property Filter: string read FFilter write FFilter;
    property StringsCase: TStringsCase read FStringsCase write SetStringsCase;
    property Align;
    property BorderStyle;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnFileChange: TNotifyEvent read FOnFileChange write FOnFileChange;
    property OnDirChange: TNotifyEvent read FOnDirChange write FOnDirChange;
  end;

procedure Register;

implementation

{$R EXPLORER}

function GetBitmapFromIcon(Icon: TIcon; BgColor: TColor): TBitmap;
var
  IWidth, IHeight: Integer;
begin
  IWidth := Icon.Width;
  IHeight := Icon.Height;
  Result := TBitmap.Create;
  try
    Result.Width := IWidth;
    Result.Height := IHeight;
    with Result.Canvas do begin
      Brush.Color := BgColor;
      FillRect(Rect(0, 0, IWidth, IHeight));
      Draw(0, 0, Icon);
    end;
  except
    Result.Free;
  end;
end;

constructor TExplorerBox.Create(AOwner: TComponent);
var
  i: Integer;
begin
  inherited Create(AOwner);
  Style := lbOwnerDrawFixed;
  Columns := 1;
  FFilter := '*.*';
  FStringsCase := cs_Normal;
  for i := 0 to 3 do FBitmaps[i] := TBitmap.Create;
  FBitmaps[0].Handle := LoadBitmap(HInstance, 'BITMAP_1');
  FBitmaps[1].Handle := LoadBitmap(HInstance, 'BITMAP_2');
  FBitmaps[2].Handle := LoadBitmap(HInstance, 'BITMAP_3');  
  FIcon := TIcon.Create;
end;

destructor TExplorerBox.Destroy;
var
  i: Integer;
begin
  for i := 0 to 3 do FBitmaps[i].Free;
  inherited Destroy;
end;

function TExplorerBox.RPos(const SubStr: Char; Str: string): Word;
var
  i: Word;
begin
  Result := 0;
  repeat
    if Str[Length(Str)] = SubStr then exit else
    begin
      Dec(Str[0]);
      Inc(Result)
    end
  until Length(Str) = 0;
end;

procedure TExplorerBox.SetDirectory(Value: TFileName);
begin
  if Value[Length(Value)] <> '\' then
    FDirectory := Value + '\'
  else
    FDirectory := Value;
  Update;
end;

procedure TExplorerBox.Update;
var
  ARec: TSearchRec;
  Res, Index: Integer;
  SList: TStringList;

begin
  Clear;
  SList := TStringList.Create;
  SList.Sorted := Sorted;

  Res := FindFirst(FDirectory + '*.*', faArchive or faDirectory, ARec);

  repeat
      case FStringsCase of
        cs_Upper: ARec.Name := UpperCase(ARec.Name);
        cs_Lower: ARec.Name := LowerCase(ARec.Name);
      end;

      if (ARec.Attr and faDirectory) > 0 then
               if ARec.Name <> '.' then Items.Add(ARec.Name);


      if (ARec.Attr and faArchive) > 0 then
      begin
        if FFilter = '*.*' then SList.Add(ARec.Name) 
             else
        if Pos(ExtractFileExt(ARec.Name), FFilter) <> 0 then SList.Add(ARec.Name)
      end;

      Res := FindNext(ARec);
  Until Res <> 0;

  for Index := 0 to SList.Count - 1 do
        Items.Insert(Items.Count, SList.Strings[Index]);

  SList.Free;
end;

procedure TExplorerBox.Click;
var
  Res: Integer;
  AFull: string;
begin
  inherited Click;
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if Res = faArchive then
  begin
    if Assigned(FOnFileChange) then FOnFileChange(Self);
    FFileName := FDirectory + Items.Strings[ItemIndex]
  end
end;

procedure TExplorerBox.DblClick;
var Res: Integer;
    DirOld, AFull: string;
begin
  if ItemIndex = -1 then Exit;
  if Items.Strings[ItemIndex] = '..' then
  begin
    Dec(FDirectory[0]);
    AFull := Copy(FDirectory, 0, Length(FDirectory) - RPos('\', FDirectory));
    DirOld := Copy(FDirectory, Length(FDirectory) - RPos('\', FDirectory) + 1,
                   Length(FDirectory));
    Directory := AFull;
    if Assigned(FOnDirChange) then FOnDirChange(Self);
    ItemIndex := Items.IndexOf(DirOld);
    exit
  end;
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if (Res AND faDirectory) > 0 then
  begin
    Directory := AFull;
    if Assigned(FOnDirChange) then FOnDirChange(Self);
    ItemIndex := 0
  end else inherited DblClick
end;

procedure TExplorerBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = 13 then DblClick;
end;

procedure TExplorerBox.RePaint;
begin
  SetDirectory(FDirectory);
  inherited RePaint
end;

procedure TExplorerBox.DrawItem(Index: Integer; Recty: TRect; State: TOwnerDrawState);
var
  i, Res: Integer;
  PFileName: array [0..255] of Char;
  IcnRecty: TRect;
begin
  with Canvas do
  begin
    FillRect(Recty);

    Res := FileGetAttr(FDirectory + Items.Strings[Index]);

    if (Res and faDirectory) > 0 then
    begin
      if Items.Strings[Index] = '..' then
      begin
        inherited DrawItem(Index, Recty, State);
        exit;
      end;
      Draw(Recty.Left, Recty.Top, FBitmaps[0]);
      TextOut(Recty.Left + 18, Recty.Top, Items[Index]);
    end else

    if Pos('.exe', LowerCase(Items[Index])) <> 0 then
    begin
      StrPCopy(PFileName, FDirectory + Items.Strings[Index]);
      FIcon.Handle := ExtractIcon(Application.Handle, PFileName, 0);

      if (FIcon.Handle <> 0) AND (FIcon.Handle <> 1) then
      begin
        FBitmaps[3] := GetBitmapFromIcon(FIcon, clWhite);
        IcnRecty := Rect(Recty.Left, Recty.Top, Recty.Left + 14, Recty.Top + 14);
        StretchDraw(IcnRecty, FBitmaps[3]);

        TextOut(Recty.Left + 18, Recty.Top, Items[Index]);
      end else begin
            Draw(Recty.Left, Recty.Top, FBitmaps[2]);
            TextOut(Recty.Left + 18, Recty.Top, Items[Index]);
          end;
    end else begin
      Draw(Recty.Left, Recty.Top, FBitmaps[1]);
      TextOut(Recty.Left + 18, Recty.Top, Items[Index]);
    end;
  end;
end;

procedure TExplorerBox.CreateWnd;
var
  s: string;
begin
  inherited CreateWnd;
  if Directory = '' then
  begin
    GetDir(0, s);
    Directory := s
  end;
end;

procedure TExplorerBox.SetStringsCase(ACase: TStringsCase);
begin
  FStringsCase := ACase;
  Update;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TExplorerBox]);
end;

end.
